/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton and J. Kevin Etheridge
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef __WBMP__
#include "WBmp.h"
#endif
	
#include <stdio.h>

WBmp::WBmp(unsigned long width, unsigned long height)
{
	mWidth = width;
	mHeight = height;
	mWindow = NULL;

	BITMAPINFO bmi;
	memset(&bmi, 0, sizeof(BITMAPINFO));
	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmi.bmiHeader.biHeight = -1 * (long)height; // negative
	bmi.bmiHeader.biWidth = width;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biBitCount = 32;


	mDC = CreateCompatibleDC(NULL); // create a dc compatible with the screen

	// sidestepping the error here on purpose
	if(!mDC)
		mBitmap = NULL;
	else
		mBitmap = CreateDIBSection(mDC, &bmi, DIB_RGB_COLORS, &mBuffer, NULL, 0);
	
}
WBmp::~WBmp()
{
	if(mDC)
		DeleteDC(mDC);
	if(mBitmap)
		DeleteObject(mBitmap);
}
void WBmp::SetWindow(HWND hwnd)
{
	mWindow = hwnd;
}

BOOL WBmp::WriteToFile(char *filename)
{
	char bmpfileheader[14];
	char bmpinfoheader[40];
	int headersize, bfSize;
	int bits_per_pixel, cmap_entries;

#define PUT_2B(array,offset,value)  \
	(array[offset] = (char) ((value) & 0xFF), \
	 array[offset+1] = (char) (((value) >> 8) & 0xFF))
#define PUT_4B(array,offset,value)  \
	(array[offset] = (char) ((value) & 0xFF), \
	 array[offset+1] = (char) (((value) >> 8) & 0xFF), \
	 array[offset+2] = (char) (((value) >> 16) & 0xFF), \
	 array[offset+3] = (char) (((value) >> 24) & 0xFF))


    // Unquantized, full color RGB 
	bits_per_pixel = 24;
	cmap_entries = 0;

	// File size 
	headersize = 14 + 40 + cmap_entries * 4; // Header and colormap 
	bfSize = headersize + (int) mWidth * (int) mHeight;
  
	// Set unused fields of header to 0 
	memset(bmpfileheader, 0, sizeof(bmpfileheader));
	memset(bmpinfoheader, 0, sizeof(bmpinfoheader));

	// Fill the file header 
	bmpfileheader[0] = 0x42;	/* first 2 bytes are ASCII 'B', 'M' */
	bmpfileheader[1] = 0x4D;
	PUT_4B(bmpfileheader, 2, bfSize); // bfSize 
	// we leave bfReserved1 & bfReserved2 = 0 
	PUT_4B(bmpfileheader, 10, headersize); // bfOffBits 

	// Fill the info header (Microsoft calls this a BITMAPINFOHEADER) 
	PUT_2B(bmpinfoheader, 0, 40);	// biSize 
	PUT_4B(bmpinfoheader, 4, mWidth); // biWidth 
	PUT_4B(bmpinfoheader, 8, mHeight); // biHeight 
	PUT_2B(bmpinfoheader, 12, 1);	// biPlanes - must be 1 
	PUT_2B(bmpinfoheader, 14, bits_per_pixel); // biBitCount 

	// we leave biCompression = 0, for none 
	// we leave biSizeImage = 0; this is correct for uncompressed data 
	// leave density at zero 
	PUT_2B(bmpinfoheader, 32, cmap_entries); // biClrUsed 
	// we leave biClrImportant = 0 


	FILE *out = NULL;
	BOOL success = false;

	unsigned char *temp = NULL;
	unsigned long rowbytes = mWidth * 3; 
	unsigned long y;

	if(!(out = fopen(filename, "wb")))
		goto done;

	if(!fwrite( bmpfileheader, 14, 1, out ))
		goto done;

	if(!fwrite( bmpinfoheader, 40, 1, out ))
		goto done;

	// make rowbytes a multiple of four
	if(rowbytes & 0x3)
		rowbytes+= (4 - rowbytes & 0x3);

	temp = new unsigned char[rowbytes+1]; // since I'm setting four bytes over three, need one extra here
	if(!temp)
		goto done;

	// memset to account for padding
	memset(temp, 0, rowbytes);


	// loop through rows
	for(y = 0; y < mHeight; y++)
	{
		int x = mWidth;
		unsigned char *src = (unsigned char *)GetRow(mHeight - 1 - y);
		unsigned char *dst = temp;

		// loop through columns
		while(x--)
		{			
			// ditch the padding
			*((DWORD *)dst) = *((DWORD *)src);
			dst += 3;
			src += 4;
		}

		// write our 24 bit row
		if(!fwrite( temp , rowbytes , 1, out ))
			goto done;

	}

	success = true;

done:

	if(temp)
		delete [] temp;
	if(out)
		fclose(out);
	return success;

}




// returns false on error
BOOL WBmp::ReadFromFile(const char *inFileName)
{

	int offBits ;
	char *p = NULL;
		
	HANDLE hFile = NULL, hFileMap = NULL;
	char *pFile = NULL;
    WORD nNumColors;
	DWORD dwSize;
    LPBITMAPINFOHEADER lpbi;
    BITMAPFILEHEADER bf;
	BITMAPINFO *bmip;
	
	static HANDLE heap;

	// May want to replace this with decent memory allocation
	// allocate the initial room
	p = (char *)HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, (DWORD)(sizeof(BITMAPINFOHEADER) + 256 * sizeof( RGBQUAD))) ;
	if( !p )
		goto fail;

	hFile = CreateFile(	inFileName,
					GENERIC_READ,
					0,
					NULL,
					OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN,
					NULL) ;

	if( hFile == INVALID_HANDLE_VALUE)
	{
		hFile = NULL;
		goto fail;
	}

	dwSize = GetFileSize( hFile, NULL) ;// get the file size
	// create file mapping
	hFileMap = CreateFileMapping( hFile, NULL, PAGE_READONLY, 0, dwSize, NULL) ;
	if(!hFileMap)
		goto fail;

	pFile = (char *)MapViewOfFile( hFileMap, FILE_MAP_READ, 0, 0, 0) ;
	if(!pFile)
		goto fail;
	
	// point to the beginning of the memory block
	lpbi = (LPBITMAPINFOHEADER)p ;

	// copy memory in the BITMAPFILEHEADER data structure
	CopyMemory( &bf, pFile, sizeof( BITMAPFILEHEADER)) ;

	// 'BM'
	if( bf.bfType != 0x4d42)
		goto fail;

	// copy memory in the BITMAPINFOHEADER data structure
	CopyMemory( lpbi, pFile + sizeof( BITMAPFILEHEADER), sizeof( BITMAPINFOHEADER)) ;

	if( lpbi -> biSize == sizeof( BITMAPCOREHEADER))
		goto fail;

	if( !( nNumColors = (WORD)lpbi -> biClrUsed))
	{ 
		// dimensioning the color table
		if( lpbi -> biBitCount != 24)
	    	nNumColors = 1 << lpbi->biBitCount ;
	}
	// filling the structure
	if( lpbi -> biClrUsed == 0)
		lpbi -> biClrUsed = (DWORD)nNumColors ;
	if( lpbi->biSizeImage == 0)
		lpbi->biSizeImage = ((((lpbi->biWidth * (DWORD)lpbi->biBitCount) + 31) & ~31) >> 3) * lpbi->biHeight ;
	
	// buffer for the header, color table, and the data
	p = (char *)HeapReAlloc( GetProcessHeap(), 0, p, lpbi->biSize + nNumColors * sizeof(RGBQUAD) + lpbi -> biSizeImage) ;
	if( !p )
		goto fail;

	lpbi = (LPBITMAPINFOHEADER)p ;

	// reading the color table
	CopyMemory( (LPSTR)(lpbi) + lpbi -> biSize,  pFile + sizeof( BITMAPFILEHEADER) + sizeof( BITMAPINFOHEADER), nNumColors * sizeof(RGBQUAD)) ;

	// offset from the beginning of the DIB header
	offBits = (WORD)lpbi -> biSize + nNumColors * sizeof( RGBQUAD) ;

	CopyMemory( (LPSTR)lpbi + offBits, pFile + sizeof( BITMAPFILEHEADER) + sizeof( BITMAPINFOHEADER) + nNumColors * sizeof(RGBQUAD), lpbi->biSizeImage) ;

	
	UnmapViewOfFile( pFile) ;
	pFile = NULL;
	(void)CloseHandle( hFileMap);
	hFileMap = NULL;
	(void)CloseHandle( hFile);
	hFile = NULL;


	bmip = (BITMAPINFO *)p;

	Resize(bmip->bmiHeader.biWidth,bmip->bmiHeader.biHeight);

	SelectObject(mDC,mBitmap);

 	if(!SetDIBitsToDevice(
		mDC,	// handle of device context 
		0,	// x-coordinate of upper-left corner of dest. rect. 
		0,	// y-coordinate of upper-left corner of dest. rect. 
		bmip->bmiHeader.biWidth,	// source rectangle width 
		bmip->bmiHeader.biHeight,	// source rectangle height 
		0,	// x-coordinate of lower-left corner of source rect. 
		0,	// y-coordinate of lower-left corner of source rect. 
		0,	// first scan line in array 
		bmip->bmiHeader.biHeight,	// number of scan lines 
		(char *)bmip->bmiColors + bmip -> bmiHeader.biClrUsed*sizeof(RGBQUAD),	// address of array with DIB bits 
		bmip,	// address of structure with bitmap info. 
		DIB_RGB_COLORS
	   ) )
	{
		goto fail;
	}

	HeapFree(GetProcessHeap(),0,p);
	return true;


fail:
	if(pFile)
		UnmapViewOfFile(pFile);
	if(hFileMap)
		CloseHandle(hFileMap);
	if(hFile)
		CloseHandle(hFile);
	if(p)
		HeapFree(GetProcessHeap(),0,p);


	return false;
}





void WBmp::Draw(long inX, long inY, long inSrcX, long inSrcY, long width, long height, HDC inHdc)
{
	HDC hdc;
	BOOL isHalftone;

	// make sure we're allocated
	if(!mBitmap)
		return;
	
	if(inHdc)
	{
		hdc = inHdc;
	}
	else
	{
		// Get the device context of the window we're about to paint into
		hdc = GetDC(mWindow);
		if(!hdc) 
			return;
	}

	// determine if we're in a halftone mode
	isHalftone = (GetDeviceCaps(hdc, NUMCOLORS) > 0);

	// if the width and height are zero, go to the default of the whole image
	if(!width) width = mWidth;
	if(!height) height = mHeight;


	// select bitmap into our offscreen dc
	mBitmap = (HBITMAP)SelectObject(mDC, mBitmap);
	if(isHalftone)
	{
		HPALETTE pal;
		SetStretchBltMode(hdc, STRETCH_HALFTONE);
		pal = CreateHalftonePalette(hdc);
		if(pal)
		{
			pal = SelectPalette(hdc, pal, TRUE);
			RealizePalette(hdc);
			(void)BitBlt(hdc, inX, inY, width, height, mDC, inSrcX, inSrcY, SRCCOPY);
			pal = SelectPalette(hdc, pal, TRUE);
			DeleteObject(pal);
		}
	}
	else
	{
		// blit to the screen
		(void)BitBlt(hdc, inX, inY, width, height, mDC, inSrcX, inSrcY, SRCCOPY);
	}

	// get our bitmap out of the offscreen dc
	mBitmap = (HBITMAP)SelectObject(mDC, mBitmap);
	// release the device context of the window we're painting into
	if(!inHdc)
		ReleaseDC(mWindow, hdc);

}


void WBmp::Resize(unsigned long newWidth, unsigned long newHeight)
{
	if(newWidth == mWidth && newHeight == mHeight)
		return; // no need to resize if we're already at the right size

	mWidth = newWidth;
	mHeight = newHeight;

	// destroy the old bitmap
	if(mBitmap)
	{
		DeleteObject(mBitmap);
		mBitmap = NULL;
	}

	BITMAPINFO bmi;
	memset(&bmi, 0, sizeof(BITMAPINFO));
	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmi.bmiHeader.biHeight = -1 * (long)mHeight; // negative
	bmi.bmiHeader.biWidth = mWidth;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biBitCount = 32;

	// ignoring error here on purpose
	if(mDC)
		mBitmap = CreateDIBSection(mDC, &bmi, DIB_RGB_COLORS, &mBuffer, NULL, 0);
}

unsigned long WBmp::MakeLongColor(unsigned char red, unsigned char green, unsigned char blue)
{
	return blue | ((int)green << 8) | ((int)red << 16);
}

// make the bitmap one solid color
void WBmp::Solid(unsigned long color)
{
	if(mBitmap)
	{
		unsigned long *p = (unsigned long *)mBuffer;
		unsigned long i = mWidth * mHeight;
		while(i--)
			*p++ = color;	
	}
}


// make a copy of a bitmap resource
void WBmp::CopyResource(HINSTANCE inInstance, UINT id)
{
	HBITMAP hBitmap;

	// load the resource
	hBitmap = (HBITMAP)LoadImage(inInstance, 
		MAKEINTRESOURCE(id), IMAGE_BITMAP, 0,0, LR_CREATEDIBSECTION);

	if(!hBitmap)
		return;

	// get information
	BITMAP bitmap;
	if(!GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bitmap))
	{
		DeleteObject(hBitmap);
		return;
	}

	// change to new size
	Resize(bitmap.bmWidth, bitmap.bmHeight);
	if((int)mWidth != bitmap.bmWidth || (int)mHeight != bitmap.bmHeight)
	{
		DeleteObject(hBitmap);
		return;
	}

	HDC hdc = CreateCompatibleDC(mDC);
	if(!hdc)
	{
		DeleteObject(hBitmap);
		return;
	}


	// select the bitmaps into the device contexts
	mBitmap = (HBITMAP)SelectObject(mDC, mBitmap);
	hBitmap = (HBITMAP)SelectObject(hdc, hBitmap);

	// blit
	(void)BitBlt(mDC, 0, 0, mWidth, mHeight, hdc, 0, 0, SRCCOPY);

	// get our bitmap out of the offscreen dc
	mBitmap = (HBITMAP)SelectObject(mDC, mBitmap);
	hBitmap = (HBITMAP)SelectObject(hdc, hBitmap);
	// free the image
	DeleteObject(hBitmap);
}




