/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton and J. Kevin Etheridge
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __WBMP__
#define __WBMP__

#include <windows.h>

class WBmp {
public:
	WBmp(unsigned long width=1, unsigned long height=1);
	~WBmp();

	// call this function to set the display window for this bitmap
	void SetWindow(HWND hwnd);

	// draw bitmap into window
	// width and height, if zero, will default to actual width and height of the image
	void Draw(long inX = 0, long inY = 0, long inSrcX = 0, long inSrcY = 0, long width = 0, long height = 0, HDC inHdc = NULL);

	// change the size of the bitmap
	// does not actually do any data 'stretching' or sizing, only changes the size of the buffer
	void Resize(unsigned long newWidth, unsigned long newHeight);
	
	// make the bitmap one solid color
	void Solid(unsigned long color);
	
	// make a copy of a bitmap resource
	void CopyResource(HINSTANCE inInstance, UINT id);
	
	// make a 32 bit color for use with this bitmap class
	static unsigned long MakeLongColor(unsigned char red, unsigned char green, unsigned char blue);
	// get a pointer to the offscreen buffer for direct manipulation
	void *GetBuffer(){return mBuffer;}
	void *GetRow(long rowNumber){ return (void *)( ((unsigned char *)GetBuffer()) + GetWidth() * 4 * rowNumber);}

	unsigned long GetWidth(){return mWidth;}
	unsigned long GetHeight(){return mHeight;}

	// writes the bitmap to a file
	// false on error
	BOOL WriteToFile(char *filename);

	// read bitmap from a file.
	// returns false on error
	BOOL ReadFromFile(const char *inFileName);


protected:
	unsigned long mWidth, mHeight;


	HDC mDC; // device context compatible with the screen
	HBITMAP mBitmap; // bitmap handle
	void *mBuffer; // buffer to hold the bitmap raster data
	HWND mWindow; // window this bitmap is to be displayed in
};


#endif

