/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __TRANSFORM__
#define __TRANSFORM__

typedef long Pixel;

typedef struct {
	double m[3][3]; // in row, col order of course.
} Matrix3By3;

typedef double Point3[3];


// This builds an identity matrix.
void IdentityMatrix( Matrix3By3& outMatrix );


// This multiplies two matrices.
void MatrixMultiply( const Matrix3By3& a, const Matrix3By3& b, Matrix3By3& out );

// This builds a translation matrix.
void TranslationMatrix( Matrix3By3& outMatrix, double xOffset, double yOffset );

// This builds a rotation matrix(including pivoting around a fixed point instead of 0,0)
void RotationMatrix( Matrix3By3& outMatrix, double xCenter, double yCenter, double angle );


// This builds a scaling matrix.
void ScalingMatrix( Matrix3By3& outMatrix, 
	double xCenter, double yCenter, 
	double xScale, double yScale );


// This maps an image to into another image using an inverse transformation,
// which means the matrix you pass should find the coordinate in the source
// given the coordinate in the destination.
void InvTransformImage( 
	const Matrix3By3& invMatrix,
	
	const unsigned char *src,
	unsigned long srcWidth,
	unsigned long srcHeight,
	unsigned long srcRowBytes,
	
	unsigned char *dest,
	unsigned long destWidth, // these are for clipping.
	unsigned long destHeight,
	unsigned long destRowBytes,
	
	// these specify the rect we loop through on the dest points
	// doing the inverse transformation,
	// have to pass it since it depends entirely on what the hell
	// you put into invMatrix
	unsigned long loopLeft,
	unsigned long loopTop,
	unsigned long loopRight,
	unsigned long loopBottom
	);


#endif 
