/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton and J. Kevin Etheridge
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __WINUTILS__
#include "winutils.h"
#endif

// returns pointer to null terminator of string if not found
// if found, returns pointer to '.'
char *GetFileExtension(char *inFileName)
{
	char *dot = NULL;


	while(*inFileName)
	{
		switch(*inFileName)
		{
		case ':':
		case '\\':
		case '//':
			// if there was a dot already, it had nothing to do with the
			// filename, and something to do with a folder name or something
			dot = NULL; 
			break;
		case '.':
			dot = inFileName;
		default:
			break;
		}
		inFileName++;
	}

	if(!dot)
		dot = inFileName + strlen(inFileName);

	return dot;

}


// get rect of a child window in coordinates of parent window
bool GetChildWindowRect(HWND parent, HWND child, RECT *outRect)
{
	RECT r;
	POINT pt;
	if( !GetWindowRect(child,&r) )
		return false;
	
	pt.x = r.left;
	pt.y = r.top;
	if( !ScreenToClient(parent,&pt) )
		return false;
	r.left = pt.x;
	r.top = pt.y;

	pt.x = r.right;
	pt.y = r.bottom;
	if( !ScreenToClient(parent,&pt) )
		return false;
	r.right = pt.x;
	r.bottom = pt.y;

	*outRect = r;
	return true;
}





// dialog hook for the open dialog
// using this hook to make sure the save dialog is centered correctly
static UINT APIENTRY OFNHookProc( HWND hwnd,UINT msg, WPARAM wParam, LPARAM lParam )
{
	if(msg == WM_INITDIALOG)
	{
		// to get this thing centered in the screen, get the parent of the passed window
		hwnd = GetParent(hwnd);

		RECT parentRect;
		RECT windowRect;


		HWND parent = GetParent(hwnd);

		// get the dimensions of this dialog
		GetWindowRect(hwnd, &windowRect);

		// get the dimensions of the parent
		if(!parent)
		{
			parentRect.top = 0;
			parentRect.left = 0;
			parentRect.bottom = GetSystemMetrics(SM_CYFULLSCREEN);
			parentRect.right = GetSystemMetrics(SM_CXFULLSCREEN);
		}
		else
		{
			GetWindowRect(GetParent(hwnd), &parentRect);
		}

		int x = 0, y = 0;

		if(parentRect.right - parentRect.left > windowRect.right - windowRect.left)
			x = parentRect.right - parentRect.left - (windowRect.right - windowRect.left);

		if(parentRect.bottom - parentRect.top > windowRect.bottom - windowRect.top)
			y = parentRect.bottom - parentRect.top - (windowRect.bottom - windowRect.top);


		// center horizontally
		x = x / 2;
		// want the dialog more skewed toward the top
		y = y / 3;

		// set the new position to center the window
		SetWindowPos(hwnd, NULL, x,y,0,0, SWP_NOZORDER | SWP_NOSIZE);
	}
	return false; // false indicated continue processing to the default handler
}

// get the file name to save it to
// false if user hit cancel or dialog failed
BOOL GetImageSaveFileName(HINSTANCE inInstance, char *filename, unsigned buffsize)
{
	OPENFILENAME ofn;

	char myfilter[] = "Bitmap Files\0*.BMP\0";

	filename[0] = 0;

	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = GetDesktopWindow();
	ofn.hInstance = inInstance;
	ofn.lpstrFilter = myfilter;

	ofn.lpstrCustomFilter = NULL;  // for custom filters
	ofn.nMaxCustFilter = 0;        // ignored

	// set index to the last used file type
	ofn.nFilterIndex = 1;

	ofn.lpstrFile = filename;
	ofn.nMaxFile = buffsize;

    ofn.lpstrFileTitle = NULL; // if we want the filename without full path
    ofn.nMaxFileTitle = 0; // ignored

    ofn.lpstrInitialDir = NULL; // tells system to use current directory
    ofn.lpstrTitle = NULL; // title of the dialog box, null for default

    ofn.Flags = (OFN_HIDEREADONLY | OFN_ENABLEHOOK) | OFN_EXPLORER;// | OFN_OVERWRITEPROMPT;
    ofn.nFileOffset = 0; //
    ofn.nFileExtension = 0; //
    ofn.lpstrDefExt = NULL;// i can put a default extension here, or I can do it after the fact or in a hook
    ofn.lCustData = NULL;
    ofn.lpfnHook = OFNHookProc; // put in our hook
    ofn.lpTemplateName = NULL;

	if(GetSaveFileName( &ofn))
	{
		char *ext = GetFileExtension(filename);
		if(!(*ext) &&  // look to see if there is no extension
			(strlen(filename) + 5 < buffsize)  // make sure there is room to attach an extension
			)
		{
			ext = ".BMP";
			strcat(filename, ext);
		}
		return true;
	}
	else
	{
		return false;
	}
}



// get the file name to open
// false if user hit cancel or dialog failed
BOOL GetImageOpenFileName(HINSTANCE inInstance, char *filename, unsigned buffsize)
{
	OPENFILENAME ofn;

	char myfilter[] = "Bitmap Files\0*.BMP\0";

	filename[0] = 0;

	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = GetDesktopWindow();
	ofn.hInstance = inInstance;
	ofn.lpstrFilter = myfilter;

	ofn.lpstrCustomFilter = NULL;  // for custom filters
	ofn.nMaxCustFilter = 0;        // ignored

	// set index to the last used file type
	ofn.nFilterIndex = 1;

	ofn.lpstrFile = filename;
	ofn.nMaxFile = buffsize;

    ofn.lpstrFileTitle = NULL; // if we want the filename without full path
    ofn.nMaxFileTitle = 0; // ignored

    ofn.lpstrInitialDir = NULL; // tells system to use current directory
    ofn.lpstrTitle = NULL; // title of the dialog box, null for default

    ofn.Flags = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_ENABLEHOOK | OFN_EXPLORER;// | OFN_OVERWRITEPROMPT;
    ofn.nFileOffset = 0; //
    ofn.nFileExtension = 0; //
    ofn.lpstrDefExt = NULL;// i can put a default extension here, or I can do it after the fact or in a hook
    ofn.lCustData = NULL;
    ofn.lpfnHook = OFNHookProc; // put in our hook
    ofn.lpTemplateName = NULL;

	if(GetOpenFileName( &ofn))
	{
		char *ext = GetFileExtension(filename);
		if(!(*ext) &&  // look to see if there is no extension
			(strlen(filename) + 5 < buffsize)  // make sure there is room to attach an extension
			)
		{
			ext = ".BMP";
			strcat(filename, ext);
		}
		return true;
	}
	else
	{
		return false;
	}
}
